/*
 * clock.c - Set up clocks for EMMA3TJ ALPHA14 evaluation board.
 *
 * Copyright (C) 2010 Renesas Electronics Corporation
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/list.h>
#include <linux/clk.h>

#include <asm/io.h>
#include <asm/emma/clock.h>
#include <asm/emma/emmaxxx.h>

#define CTOP_BASE		0x10111200
#define CTOP_STCCLK_SEL		0x190	 /* STC Clock Slect Register */

#define IICCLK_SEL		0x30000000

static void clk_i2c_enable(struct clk *clk)
{
}

static void clk_i2c_disable(struct clk *clk)
{
}

static unsigned long clk_i2c_getrate(struct clk *clk)
{
	void __iomem *ctop;
	unsigned long rate;
	u32 stcclk_sel;

	ctop = ioremap(CTOP_BASE, 0x200);
	stcclk_sel = readl(ctop + CTOP_STCCLK_SEL);
	iounmap(ctop);

	switch ((stcclk_sel & IICCLK_SEL) >> 28) {
	case 0:
		rate = 24000000;
		break;
	case 2:
	case 3:
		rate = 24000000; /* PRCG(0), PRCG(10) reset default */
		break;
	case 1:
	default:
		rate = 81000000;
		break;
	}
	return rate;
}

static struct clkops clk_i2c_ops = {
	.enable		= clk_i2c_enable,
	.disable	= clk_i2c_disable,
	.getrate	= clk_i2c_getrate,
};

static DEFINE_CLK(i2c, &clk_i2c_ops, 81000000);

static struct clk_lookup lookups[] = {
	INIT_CLKREG(&clk_i2c, "i2c_designware.0", NULL),
	INIT_CLKREG(&clk_i2c, "i2c_designware.1", NULL),
	INIT_CLKREG(&clk_i2c, "i2c_designware.2", NULL),
};

static int __init clk_init(void)
{
	clks_register(lookups, ARRAY_SIZE(lookups));
	return 0;
}
arch_initcall(clk_init);
