/*
 *  This file defines the irq handler for ALPHA14.
 *
 * Copyright (C) 2010 Renesas Electronics Corporation
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
#include <linux/init.h>
#include <linux/interrupt.h>
#include <linux/irq.h>

#include <asm/mipsregs.h>
#include <asm/irq_cpu.h>
#include <asm/emma/emmaxxx.h>
#include <asm/emma/irq.h>
#include <asm/emma/bhif.h>

/*
 * IRQ mapping
 *
 *  0-7: 8 CPU interrupts
 *	0 -	software interrupt 0
 *	1 - 	software interrupt 1
 *	2 - 	most Vrc5477 interrupts are routed to this pin
 *	3 - 	(optional) some other interrupts routed to this pin for debugg
 *	4 - 	not used
 *	5 - 	not used
 *	6 - 	not used
 *	7 - 	cpu timer (used by default)
 *
 */

void emma_irq_dispatch(void)
{
	u32 status;
	u32 bitmask;
	u32 i;

	status = emma_in32(EMMA_BHIF_INT_ST_0) &
		 emma_in32(EMMA_BHIF_INT_EN_0);

#ifdef EMMA_SW_CASCADE
	if (status & (1UL << ((EMMA_SW_CASCADE - EMMA_IRQ_BASE) % 32))) {
		u32 sw_int_status;

		sw_int_status = emma_in32(EMMA_BHIF_SW_INT) &
				emma_in32(EMMA_BHIF_SW_INT_EN);
		for (i = 0, bitmask = 1UL; i < 32; i++, bitmask <<= 1) {
			if (sw_int_status & bitmask) {
				do_IRQ(EMMA_SW_IRQ_BASE + i);
				return;
			}
		}
	}
#endif

	for (i = 0, bitmask = 1UL; i < 32; i++, bitmask <<= 1) {
		if (status & bitmask) {
			do_IRQ(EMMA_IRQ_BASE + i);
			return;
		}
	}

	status = emma_in32(EMMA_BHIF_INT_ST_1) &
		 emma_in32(EMMA_BHIF_INT_EN_1);

	for (i = 32, bitmask = 1UL; i < 64; i++, bitmask <<= 1) {
		if (status & bitmask) {
			do_IRQ(EMMA_IRQ_BASE + i);
			return;
		}
	}

	status = emma_in32(EMMA_BHIF_INT_ST_2) &
		 emma_in32(EMMA_BHIF_INT_EN_2);

	for (i = 64, bitmask = 1UL; i < 96; i++, bitmask <<= 1) {
		if (status & bitmask) {
			do_IRQ(EMMA_IRQ_BASE + i);
			return;
		}
	}
}

void __init arch_init_irq(void)
{
	/* by default, interrupts are disabled. */
	emma_out32(EMMA_BHIF_INT_EN_0, 0);
	emma_out32(EMMA_BHIF_INT_EN_1, 0);
	emma_out32(EMMA_BHIF_INT_EN_2, 0);
	emma_out32(EMMA_BHIF_SW_INT_EN, 0);

	clear_c0_status(0xff00);
	set_c0_status(0x0400);

	/* init all controllers */
	mips_cpu_irq_init();
	bhif_irq_init();
	bhif_sw_irq_init();

	/* setup cascade interrupts */
	setup_irq(MIPS_CPU_IRQ_BASE + 2, &irq_cascade);
	setup_irq(EMMA_SW_CASCADE, &irq_cascade);
}
