/*
 *  This file sets up platform devices for EMMA3TJ ALPHA14.
 *
 * Copyright (C) 2010 Renesas Electronics Corporation
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/ioport.h>
#include <linux/irq.h>
#include <linux/platform_device.h>
#include <linux/dma-mapping.h>
#include <linux/mtd/physmap.h>
#include <linux/mtd/partitions.h>

#include <asm/addrspace.h>
#include <asm/emma/emmaxxx.h>
#include <asm/emma/prom.h>

#include <linux/nand_platform.h>
#include <linux/nec_candy_pd.h>

static struct resource i2c_emma_resources_0[] = {
	{
		.start	= EMMA_IRQ_PIIC0,
		.end	= EMMA_IRQ_PIIC0,
		.flags	= IORESOURCE_IRQ
	}, {
		.start	= EMMA_PIIC0_BASE,
		.end	= EMMA_PIIC0_BASE + 0x1000 - 1,
		.flags	= IORESOURCE_MEM,
	}, {
		.start	= EMMA_PBRD_BASE,
		.end	= EMMA_PBRD_BASE + 0x1000 - 1,
		.flags	= IORESOURCE_MEM,
	}
};

struct resource i2c_emma_resources_1[] = {
	{
		.start	= EMMA_IRQ_PIIC1,
		.end	= EMMA_IRQ_PIIC1,
		.flags	= IORESOURCE_IRQ
	}, {
		.start	= EMMA_PIIC1_BASE,
		.end	= EMMA_PIIC1_BASE + 0x1000 - 1,
		.flags	= IORESOURCE_MEM,
	}, {
		.start	= EMMA_PBRD_BASE,
		.end	= EMMA_PBRD_BASE + 0x1000 - 1,
		.flags	= IORESOURCE_MEM,
	}
};

struct resource i2c_emma_resources_2[] = {
	{
		.start	= EMMA_IRQ_PIIC2,
		.end	= EMMA_IRQ_PIIC2,
		.flags	= IORESOURCE_IRQ
	}, {
		.start	= EMMA_PIIC2_BASE,
		.end	= EMMA_PIIC2_BASE + 0x1000 - 1,
		.flags	= IORESOURCE_MEM,
	}, {
		.start	= EMMA_PBRD_BASE,
		.end	= EMMA_PBRD_BASE + 0x1000 - 1,
		.flags	= IORESOURCE_MEM,
	}
};

static u32 speed_fast = 400;	/* Fast mode */
static u32 speed_standerd = 90;	/* nomal mode */

struct platform_device i2c_emma_devices[] = {
	[0] = {
		.name			= "i2c_designware",
		.id			= 0,
		.dev.platform_data	= &speed_standerd,	/* Khz */
		.resource		= i2c_emma_resources_0,
		.num_resources		= ARRAY_SIZE(i2c_emma_resources_0),
	},
	[1] = {
		.name			= "i2c_designware",
		.id			= 1,
		.dev.platform_data	= &speed_fast,	/* Khz */
		.resource		= i2c_emma_resources_1,
		.num_resources		= ARRAY_SIZE(i2c_emma_resources_1),
	},
	[2] = {
		.name			= "i2c_designware",
		.id			= 2,
		.dev.platform_data	= &speed_fast,	/* Khz */
		.resource		= i2c_emma_resources_2,
		.num_resources		= ARRAY_SIZE(i2c_emma_resources_2),
	},
};

static struct nec_candy_platform_data nec_candy_pdata = {
	.pmd_addr	= 0x00,
	.rmii		= 1,
	.clks		= MIIC_CLKS100,
};

static struct platform_device nec_candy_device = {
	.name			= "nec_candy",
	.id			= 0,
	.dev.platform_data	= &nec_candy_pdata,
	.num_resources		= 2,
	.resource = (struct resource[]) {
		{
			.name	= "nec_candy_regs",
			.start	= EMMA_ETHER_BASE,
			.end	= EMMA_ETHER_BASE + 0x400 - 1,
			.flags	= IORESOURCE_MEM,
		},
		{
			.name	= "nec_candy_irq",
			.start	= EMMA_IRQ_ETHER,
			.end	= EMMA_IRQ_ETHER,
			.flags	= IORESOURCE_IRQ,
		},
	},
};

/* Built-in USB Host Controller (OHCI) */

static struct resource ohci_resources[] = {
	{
		.start	= EMMA_OHCI_BASE,
		.end	= EMMA_OHCI_BASE + 0xff,
		.flags	= IORESOURCE_MEM,
	},
	{
		.start	= EMMA_IRQ_USB,
		.end	= EMMA_IRQ_USB,
		.flags	= IORESOURCE_IRQ,
	},
	{
		.start	= EMMA_OHCI_CONFIG_BASE,
		.end	= EMMA_OHCI_CONFIG_BASE + 0xff,
		.flags	= IORESOURCE_MEM,
	},
};

static u64 ohci_dmamask = DMA_BIT_MASK(32);

static struct platform_device emma_ohci = {
	.name = "emma-ohci",
	.id = 0,
	.dev = {
		.dma_mask		= &ohci_dmamask,
		.coherent_dma_mask	= DMA_BIT_MASK(32),
	},
	.num_resources = ARRAY_SIZE(ohci_resources),
	.resource = ohci_resources,
};

/* Built-in USB Host Controller (EHCI) */

static struct resource ehci_resources[] = {
	{
		.start	= EMMA_EHCI_BASE,
		.end	= EMMA_EHCI_BASE + 0xff,
		.flags	= IORESOURCE_MEM,
	},
	{
		.start	= EMMA_IRQ_USB,
		.end	= EMMA_IRQ_USB,
		.flags	= IORESOURCE_IRQ,
	},
	{
		.start	= EMMA_EHCI_CONFIG_BASE,
		.end	= EMMA_EHCI_CONFIG_BASE + 0xff,
		.flags	= IORESOURCE_MEM,
	},
};

static u64 ehci_dmamask = DMA_BIT_MASK(32);

static struct platform_device emma_ehci = {
	.name = "emma-ehci",
	.id = 0,
	.dev = {
		.dma_mask		= &ehci_dmamask,
		.coherent_dma_mask	= DMA_BIT_MASK(32),
	},
	.num_resources = ARRAY_SIZE(ehci_resources),
	.resource = ehci_resources,
};

#ifdef CONFIG_MTD_NAND
static struct mtd_partition nand_parts_0[] = {
	[0] = {
		.name		= "boot code",
		.offset		= MTDPART_OFS_APPEND,
		.size		= 0x00700000,
	},
	[1] = {
		.name		= "Root FS",
		.offset		= MTDPART_OFS_APPEND,
		.size		= 0x02000000,
	},
	[2] = {
		.name		= "Free Area",
		.offset		= MTDPART_OFS_APPEND,
		.size		= 0x0d840000,
	},
	[3] = {
		.name		= "boot env area 1",
		.offset		= MTDPART_OFS_APPEND,
		.size		= 0x00020000,
		.mask_flags	= MTD_WRITEABLE, /* force read-only */
	},
	[4] = {
		.name		= "boot env area 2",
		.offset		= MTDPART_OFS_APPEND,
		.size		= 0x00020000,
		.mask_flags	= MTD_WRITEABLE, /* force read-only */
	},
	[5] = {
		.name		= "Bad Block Table",
		.offset		= MTDPART_OFS_APPEND,
		.size		= 0x00080000,
		.mask_flags	= MTD_WRITEABLE, /* force read-only */
	},
};

static struct denali_nand_platform_data emma_nand_pdata_0 = {
	.parts			= nand_parts_0,
	.nr_parts		= ARRAY_SIZE(nand_parts_0),
	.page			= LARGE_PAGE,
};

static struct resource nand_emma_resources[] = {
	{
		.start	= EMMA_NAND_CON_BASE,
		.end	= EMMA_NAND_CON_BASE + 0x7ff,
		.flags	= IORESOURCE_MEM,
	},
	{
		.start	= EMMA_NAND_INDEX_BASE,
		.end	= EMMA_NAND_INDEX_BASE + 0xff,
		.flags	= IORESOURCE_MEM,
	},
	{
		.start	= EMMA_NAND_BRIDGE_BASE,
		.end	= EMMA_NAND_BRIDGE_BASE + 0xff,
		.flags	= IORESOURCE_MEM,
	},
	{
		.start	= EMMA_IRQ_NAND,
		.end	= EMMA_IRQ_NAND,
		.flags	= IORESOURCE_IRQ,
	}
};

static struct platform_device nand_emma_devices[] = {
	[0] = {
		.name			= "denali_nand",
		.id			= 0,
		.dev.platform_data	= &emma_nand_pdata_0,
		.resource		= nand_emma_resources,
		.num_resources		= ARRAY_SIZE(nand_emma_resources),
	},
};
#endif

static struct platform_device *devices[] = {
#ifdef CONFIG_I2C_DESIGNWARE_ENABLE_CH0
	&i2c_emma_devices[0],
#endif
#ifdef CONFIG_I2C_DESIGNWARE_ENABLE_CH1
	&i2c_emma_devices[1],
#endif
#ifdef CONFIG_I2C_DESIGNWARE_ENABLE_CH2
	&i2c_emma_devices[2],
#endif
	&emma_ohci,
	&emma_ehci,
#ifdef CONFIG_MTD_NAND
	&nand_emma_devices[0],
#endif
	&nec_candy_device,
};

static int __init platform_devices_setup(void)
{
	get_ethernet_addr(nec_candy_pdata.mac_addr);

	return platform_add_devices(devices, ARRAY_SIZE(devices));
}

arch_initcall(platform_devices_setup);
