/*
 *  This file is setup for EMMA3TJ ALPHA14.
 *
 *  Changes:
 *	Wind River Systems, Inc.
 *	- Make kgdb 8250 work properly. Jack Tan <jack.tan@windriver.com>
 *
 * Copyright (C) 2010 Renesas Electronics Corporation
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/ioport.h>
#include <linux/serial_8250.h>
#include <linux/kgdb.h>

#include <asm/addrspace.h>
#include <asm/time.h>
#include <asm/reboot.h>
#include <asm/emma/emmaxxx.h>
#include <asm/emma/flip.h>

static void et_emma3tj_machine_restart(char *command)
{
	static void (*back_to_prom) (void) = (void (*)(void))0xbfc00000;

	printk(KERN_INFO "EMMA3TJ ALPHA14 restart.\n");
	back_to_prom();
}

static void et_emma3tj_machine_halt(void)
{
	printk(KERN_INFO "EMMA3TJ ALPHA14 halted.\n");
	while (1)
		;
}

static void et_emma3tj_machine_power_off(void)
{
	printk(KERN_INFO "EMMA3TJ ALPHA14 halted. Please turn off the power.\n");
	while (1)
		;
}

static unsigned long emma_clock[2] = { 266000000, 320000000 };

static unsigned int __init detect_bus_frequency(unsigned long rtc_base)
{
	u32 reg;

	/* detect from boot strap */
	reg = emma_in32(EMMA_BHIF_STRAP_0);
	reg = (reg >> 4) & 0x1;

	return emma_clock[reg];
}

void __init plat_time_init(void)
{
	mips_hpt_frequency = detect_bus_frequency(0) / 2;
}

static void __init et_emma3tj_sio_setup(void)
{
	struct uart_port up;
	int emma_uart;

	memset(&up, 0, sizeof(up));

	up.flags	= UPF_BOOT_AUTOCONF | UPF_SKIP_TEST;
	up.iotype	= UPIO_DWAPB;
	up.regshift	= 2;
	up.uartclk	= 24000000;

	up.line		= 0;
	up.mapbase	= EMMA_PUART0_BASE + 3;
	up.membase	= ioremap_nocache(up.mapbase, 0x40 << up.regshift);
	up.irq		= EMMA_IRQ_PUART0;
	up.private_data	= (void *)(up.membase + (0x1F << up.regshift));

	early_serial_setup(&up);

	emma_uart	= 1;
	up.line		= emma_uart;
	up.mapbase	= EMMA_PUART0_BASE + 0x1000 * emma_uart + 3;
	up.membase	= ioremap_nocache(up.mapbase, 0x40 << up.regshift);
	up.irq		= EMMA_IRQ_PUART0 + emma_uart;
	up.private_data	= (void *)(up.membase + (0x1F << up.regshift));

	early_serial_setup(&up);
}

static void __init flip_init(void)
{
	struct flip_regs *flip;

	flip = ioremap(EMMA_FLIP_BASE, sizeof(struct flip_regs));

	/* For USB HCs */
	emma_out(flip->m_ctr[0], SWAP1_BYTE);
	emma_out(flip->usb_ctr,
		 PPON1_EN | OCI1_EN | PPON1 | INTB_EN | INTA_EN);
	emma_sync();

	iounmap((void __iomem *)flip);
}

void __init plat_mem_setup(void)
{
	set_io_port_base(KSEG1);

	_machine_restart = et_emma3tj_machine_restart;
	_machine_halt = et_emma3tj_machine_halt;
	pm_power_off = et_emma3tj_machine_power_off;

	/* setup resource limits */
	ioport_resource.end = ~0;

	/* Reboot on panic */
	panic_timeout = 50;

	et_emma3tj_sio_setup();
	flip_init();
}
