/*
 * BHIF: Host I/F block - bootstrap, bus arbiter, interrupt controller, etc.
 *
 * Copyright (C) NEC Electronics Corporation 2009
 * Copyright (C) 2008, 2009  Shinya Kuribayashi
 *
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 */
#include <linux/init.h>
#include <linux/irq.h>

#include <asm/emma/emmaxxx.h>
#include <asm/emma/bhif.h>

static void bhif_irq_unmask(unsigned int irq)
{
	u32 offset;
	u32 value;

	irq -= EMMA_IRQ_BASE;

	offset = EMMA_BHIF_INT_EN_0 +
		 (EMMA_BHIF_INT_EN_1 - EMMA_BHIF_INT_EN_0) * (irq / 32);
	value = emma_in32(offset);
	value |= 1UL << (irq % 32);
	emma_out32(offset, value);
}

static void bhif_irq_mask(unsigned int irq)
{
	u32 offset;
	u32 value;

	irq -= EMMA_IRQ_BASE;

	offset = EMMA_BHIF_INT_EN_0 +
		 (EMMA_BHIF_INT_EN_1 - EMMA_BHIF_INT_EN_0) * (irq / 32);
	value = emma_in32(offset);
	value &= ~(1UL << (irq % 32));
	emma_out32(offset, value);
}

struct irq_chip bhif_irq_controller = {
	.name		= "BHIF",
	.mask		= bhif_irq_mask,
	.mask_ack	= bhif_irq_mask,
	.unmask		= bhif_irq_unmask
};

void __init bhif_irq_init(void)
{
	unsigned int i;

	for (i = 0; i < NUM_EMMA_IRQ; i++)
		set_irq_chip_and_handler_name(EMMA_IRQ_BASE + i,
					      &bhif_irq_controller,
					      handle_level_irq, "level");
}

#ifdef EMMA_SW_CASCADE

static void bhif_sw_irq_unmask(unsigned int irq)
{
	u32 value;

	irq -= EMMA_SW_IRQ_BASE;

	value = emma_in32(EMMA_BHIF_SW_INT_EN);
	value |= 1UL << irq;
	emma_out32(EMMA_BHIF_SW_INT_EN, value);
}

static void bhif_sw_irq_mask(unsigned int irq)
{
	u32 value;

	irq -= EMMA_SW_IRQ_BASE;

	value = emma_in32(EMMA_BHIF_SW_INT_EN);
	value &= ~(1UL << irq);
	emma_out32(EMMA_BHIF_SW_INT_EN, value);
}

struct irq_chip bhif_sw_irq_controller = {
	.name		= "BHIF-SW",
	.mask		= bhif_sw_irq_mask,
	.mask_ack	= bhif_sw_irq_mask,
	.unmask		= bhif_sw_irq_unmask,
};

void __init bhif_sw_irq_init(void)
{
	unsigned int i;

	for (i = 0; i < NUM_EMMA_IRQ_SW; i++)
		set_irq_chip_and_handler_name(EMMA_SW_IRQ_BASE + i,
					      &bhif_sw_irq_controller,
					      handle_level_irq, "level");
}

#endif /* EMMA_SW_CASCADE */
