/*
 * GPIO: General purpose I/O -  programmable I/O ports, interrupt generation
 *
 * Copyright (C) NEC Electronics Corporation 2009
 *
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 */
#include <linux/init.h>
#include <linux/irq.h>

#include <asm/emma/emmaxxx.h>
#include <asm/emma/gpio.h>

#ifdef CONFIG_HW_HAS_PCI

static void gpio_irq_unmask(unsigned int irq)
{
	u32 offset;
	u32 value;

	irq -= EMMA_GPIO_IRQ_BASE;

	offset = EMMA_GPIO_INT_MASK + EMMA_GPIO_INT_MASK_OFFSET * (irq / 32);
	value = emma_in32(offset);
	value |= 1UL << (irq % 32);
	emma_out32(offset, value);
}

static void gpio_irq_mask(unsigned int irq)
{
	u32 offset;
	u32 value;

	irq -= EMMA_GPIO_IRQ_BASE;

	offset = EMMA_GPIO_INT_MASK + EMMA_GPIO_INT_MASK_OFFSET * (irq / 32);
	value = emma_in32(offset);
	value &= ~(1UL << (irq % 32));
	emma_out32(offset, value);
}

static void gpio_irq_ack(unsigned int irq)
{
	u32 offset;

	irq -= EMMA_GPIO_IRQ_BASE;

	offset = EMMA_GPIO_INT_ST + EMMA_GPIO_INT_ST_OFFSET * (irq / 32);
	emma_out32(offset, ~(1UL << (irq % 32)));
}

struct irq_chip gpio_irq_controller = {
	.name		= "GPIO",
	.ack		= gpio_irq_ack,
	.mask		= gpio_irq_mask,
	.unmask		= gpio_irq_unmask,
};

void __init gpio_irq_init(void)
{
	unsigned int i;

	for (i = 0; i < NUM_EMMA_IRQ_GPIO; i++)
		set_irq_chip_and_handler_name(EMMA_GPIO_IRQ_BASE + i,
					      &gpio_irq_controller,
					      handle_edge_irq, "edge");
}

#endif /* CONFIG_HW_HAS_PCI */
