/*
 * Copyright (C) 2009  NEC Electronics Corporation
 *
 * PROM library initialisation code, supports YAMON and U-Boot:
 * Copyright 2000-2001, 2006, 2008 MontaVista Software Inc.
 * Author: MontaVista Software, Inc. <source@mvista.com>
 *
 * PROM library initialisation code:
 * Copyright (C) 1999, 2000, 2004, 2005  MIPS Technologies, Inc.
 *	All rights reserved.
 *	Authors: Carsten Langgaard <carstenl@mips.com>
 *		 Maciej W. Rozycki <macro@mips.com>
 *
 * Kernel command line creation using the prom monitor (YAMON) argc/argv:
 * Carsten Langgaard, carstenl@mips.com
 * Copyright (C) 1999,2000 MIPS Technologies, Inc.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
#include <linux/init.h>
#include <linux/string.h>
#include <linux/kernel.h>

#include <asm/bootinfo.h>
#include <asm/emma/prom.h>

int prom_argc;
char **prom_argv;
char **prom_envp;

char * __init prom_getcmdline(void)
{
	return &(arcs_cmdline[0]);
}

void __init prom_init_cmdline(void)
{
	char *cp;
	int actr;

	/* Make sure arguments are available.  If not, do nothing. */
	if (prom_argc == 0 || prom_argv == NULL)
		return;

	actr = 1; /* Always ignore argv[0] */

	cp = &(arcs_cmdline[0]);
	while (actr < prom_argc) {
		strcpy(cp, prom_argv[actr]);
		cp += strlen(prom_argv[actr]);
		*cp++ = ' ';
		actr++;
	}
	if (cp != &(arcs_cmdline[0])) {
		/* get rid of trailing space */
		--cp;
		*cp = '\0';
	}
}

char * __init prom_getenv(char *envname)
{
	int index, len, yamon;

	if (!prom_envp)
		return NULL;

	index = 0;
	len = strlen(envname);
	yamon = (prom_envp[0] && strchr(prom_envp[0], '=') == NULL);

	/*
	 * Return a pointer to the given environment variable.
	 * YAMON uses "name", "value" pairs, while U-Boot uses "name=value".
	 */
	while (prom_envp[index]) {
		if (yamon) {
			if (strncmp(envname, prom_envp[index], len) == 0)
				return prom_envp[index + 1];
			index += 2;
		} else {
			if (strncmp(envname, prom_envp[index], len) == 0 &&
			    *(prom_envp[index] + len) == '=')
				return prom_envp[index] + len + 1;
			index++;
		}
	}

	return NULL;
}

void __init prom_free_prom_memory(void)
{
}

#ifdef CONFIG_EMMA_HAS_NEC_CANDY

static inline unsigned char str2hexnum(unsigned char c)
{
	if (c >= '0' && c <= '9')
		return c - '0';
	if (c >= 'a' && c <= 'f')
		return c - 'a' + 10;
	if (c >= 'A' && c <= 'F')
		return c - 'A' + 10;
	return 0; /* foo */
}

static inline void str2eaddr(unsigned char *ea, unsigned char *str)
{
	int i;

	for (i = 0; i < 6; i++) {
		unsigned char num;

		if ((*str == '.') || (*str == ':'))
			str++;
		num  = str2hexnum(*str++) << 4;
		num |= str2hexnum(*str++);
		ea[i] = num;
	}
}

int get_ethernet_addr(unsigned char *ethernet_addr)
{
	char *str;
	char *cmdline;

	/* Check the environment variables first */
	str = prom_getenv("ethaddr");
	if (!str) {
		/* Check command line */
		cmdline = prom_getcmdline();
		str = strstr(cmdline, "ethaddr=");
		if (!str) {
			printk(KERN_INFO "ethaddr is not set in boot prom\n");
			return -1;
		}

		str += strlen("ethaddr=");
	}

	str2eaddr(ethernet_addr, str);

	return 0;
}

#endif /* CONFIG_EMMA_HAS_NEC_CANDY */
