/*
 * This file is EMMA3TJ common header.
 *
 * Copyright (C) 2010 Renesas Electronics Corporation
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
#ifndef __ASM_EMMA_EMMA3TJ_H
#define __ASM_EMMA_EMMA3TJ_H

#define EMMA_BHIF_BASE			0x10000000
#define EMMA_ETHER_BASE			0x10012000
#define EMMA_PBRD_BASE			0x10100000
#define EMMA_TIMER_BASE			0x10110000
#define EMMA_FLIP_BASE			0x10200100

#define EMMA_BHIF_STRAP_0		(EMMA_BHIF_BASE + 0x0010)
#define EMMA_BHIF_ALTERNATE0		(EMMA_BHIF_BASE + 0x0028)
#define EMMA_BHIF_ALTERNATE1		(EMMA_BHIF_BASE + 0x002c)
#define EMMA_BHIF_INT_ST_0		(EMMA_BHIF_BASE + 0x0030)
#define EMMA_BHIF_INT_ST_1		(EMMA_BHIF_BASE + 0x0034)
#define EMMA_BHIF_INT_ST_2		(EMMA_BHIF_BASE + 0x0038)
#define EMMA_BHIF_INT_EN_0		(EMMA_BHIF_BASE + 0x0040)
#define EMMA_BHIF_INT_EN_1		(EMMA_BHIF_BASE + 0x0044)
#define EMMA_BHIF_INT_EN_2		(EMMA_BHIF_BASE + 0x0048)
#define EMMA_BHIF_SUB_INT_ST_0		(EMMA_BHIF_BASE + 0x0100)
#define EMMA_BHIF_SUB_INT_ST_1		(EMMA_BHIF_BASE + 0x0104)
#define EMMA_BHIF_SUB_INT_ST_2		(EMMA_BHIF_BASE + 0x0108)
#define EMMA_BHIF_SUB_INT_EN_0		(EMMA_BHIF_BASE + 0x0110)
#define EMMA_BHIF_SUB_INT_EN_1		(EMMA_BHIF_BASE + 0x0114)
#define EMMA_BHIF_SUB_INT_EN_2		(EMMA_BHIF_BASE + 0x0118)
#define EMMA_BHIF_SW_INT		(EMMA_BHIF_BASE + 0x01a0)
#define EMMA_BHIF_SW_INT_EN		(EMMA_BHIF_BASE + 0x01a4)
#define EMMA_BHIF_SW_INT_CLR		(EMMA_BHIF_BASE + 0x01a8)

#define EMMA_PUART0_BASE		(EMMA_PBRD_BASE + 0x1000)
#define EMMA_PUART1_BASE		(EMMA_PBRD_BASE + 0x2000)
#define EMMA_PIIC0_BASE			(EMMA_PBRD_BASE + 0x4000)
#define EMMA_PIIC1_BASE			(EMMA_PBRD_BASE + 0x5000)
#define EMMA_PIIC2_BASE			(EMMA_PBRD_BASE + 0x6000)

#define EMMA_GPIO0_BASE			(EMMA_TIMER_BASE + 0x0d00)
#define EMMA_GPIO0_DIR			(EMMA_GPIO0_BASE + 0xc0)
#define EMMA_GPIO0_INT			(EMMA_GPIO0_BASE + 0xc8)
#define EMMA_GPIO0_INT2			(EMMA_GPIO0_BASE + 0xcc)
#define EMMA_GPIO0_INT_CPUMEN		(EMMA_GPIO0_BASE + 0xd0)
#define EMMA_GPIO0_INT_CPUMEN_2		(EMMA_GPIO0_BASE + 0xd4)
#define EMMA_GPIO0_INT_CPUSEN		(EMMA_GPIO0_BASE + 0xd8)
#define EMMA_GPIO0_INT_CPUSEN_2		(EMMA_GPIO0_BASE + 0xdc)
#define EMMA_GPIO0_INT_MODE		(EMMA_GPIO0_BASE + 0xe0)
#define EMMA_GPIO0_INT_MODE2		(EMMA_GPIO0_BASE + 0xe4)
#define EMMA_GPIO0_INT_CND_A		(EMMA_GPIO0_BASE + 0xe8)
#define EMMA_GPIO0_INT_CND_A_2		(EMMA_GPIO0_BASE + 0xec)
#define EMMA_GPIO0_INT_CND_B		(EMMA_GPIO0_BASE + 0xf0)
#define EMMA_GPIO0_INT_CND_B_2		(EMMA_GPIO0_BASE + 0xf4)

#define EMMA_GPIO_INT_ST		EMMA_GPIO0_INT
#define EMMA_GPIO_INT_MASK		EMMA_GPIO0_INT_CPUMEN
#define EMMA_GPIO_SUB_INT_MASK		EMMA_GPIO0_INT_CPUSEN
#define EMMA_GPIO_INT_MASK_OFFSET	EMMA_GPIO_INT_ST_OFFSET

#define EMMA_OHCI_CONFIG_BASE	(EMMA_FLIP_BASE + 0x0100) /* 0x1020.0200 */
#define EMMA_EHCI_CONFIG_BASE	(EMMA_FLIP_BASE + 0x0400) /* 0x1020.0500 */
#define EMMA_EHCI_BASE		(EMMA_FLIP_BASE + 0x0e00) /* 0x1020.0F00 */
#define EMMA_OHCI_BASE		(EMMA_FLIP_BASE + 0x0f00) /* 0x1020.1000 */

#define EMMA_NAND_CON_BASE	(EMMA_FLIP_BASE + 0x2700) /* 0x1020.2800 */
#define EMMA_NAND_INDEX_BASE	(EMMA_FLIP_BASE + 0x2f00) /* 0x1020.3000 */
#define EMMA_NAND_BRIDGE_BASE	(EMMA_FLIP_BASE + 0x3000) /* 0x1020.3100 */

/*
 *  Memory map (physical address)
 *
 *  Note most of the following address must be properly aligned by the
 *  corresponding size.  For example, if PCI_IO_SIZE is 16MB, then
 *  PCI_IO_BASE must be aligned along 16MB boundary.
 */

/* the actual ram size is detected at run-time */
#define EMMA_RAM_BASE		0x00000000

#define EMMA_RAM_SIZE		0x08000000	/* 128 MB */

#define EMMA_IO_BASE		0x10000000
#define EMMA_IO_SIZE		0x01000000	/* 16 MB */

/*
 * IRQ defines
 */
#define NUM_EMMA_IRQ		96
#define NUM_EMMA_IRQ_SW		32
#define NUM_EMMA_IRQ_GPIO	48

#define EMMA_IRQ_BASE		(MIPS_CPU_IRQ_BASE + 8)
#define EMMA_SW_IRQ_BASE	(EMMA_IRQ_BASE + NUM_EMMA_IRQ)
#define EMMA_GPIO_IRQ_BASE	(EMMA_SW_IRQ_BASE + NUM_EMMA_IRQ_SW)

#define EMMA_IRQ_INT(n)		(EMMA_IRQ_BASE + (n))
#define EMMA_SW_IRQ_INT(n)	(EMMA_SW_IRQ_BASE + (n))

#define EMMA_SW_CASCADE		EMMA_IRQ_INT(6)
#define EMMA_GPIO_CASCADE	EMMA_IRQ_INT(90)

#define EMMA_IRQ_NAND		EMMA_IRQ_INT(1)
#define EMMA_IRQ_USB		EMMA_IRQ_INT(33)
#define EMMA_IRQ_ETHER		EMMA_IRQ_INT(36)
#define EMMA_IRQ_PUART0		EMMA_IRQ_INT(48)
#define EMMA_IRQ_PUART1		EMMA_IRQ_INT(49)
#define EMMA_IRQ_PIIC0		EMMA_IRQ_INT(51)
#define EMMA_IRQ_PIIC1		EMMA_IRQ_INT(52)
#define EMMA_IRQ_PIIC2		EMMA_IRQ_INT(53)

#endif	/* __ASM_EMMA_EMMA3TJ_H */
