/*
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * Copyright (C) NEC Electronics Corporation 2009
 */
#ifndef _ASM_EMMA_EMMAXXX_H
#define _ASM_EMMA_EMMAXXX_H

#ifdef CONFIG_SOC_EMMA3P
#include <asm/emma/emma3p.h>
#endif
#ifdef CONFIG_SOC_EMMA3PF
#include <asm/emma/emma3pf.h>
#endif
#ifdef CONFIG_SOC_EMMA3TJ
#include <asm/emma/emma3tj.h>
#endif

/* Processor-dependent register abstraction */
#ifdef CONFIG_EMMA_SUBCPU
#undef EMMA_BHIF_INT_ST_0
#undef EMMA_BHIF_INT_ST_1
#undef EMMA_BHIF_INT_ST_2
#undef EMMA_BHIF_INT_EN_0
#undef EMMA_BHIF_INT_EN_1
#undef EMMA_BHIF_INT_EN_2
#undef EMMA_GPIO_INT_MASK
#define EMMA_BHIF_INT_ST_0	EMMA_BHIF_SUB_INT_ST_0
#define EMMA_BHIF_INT_ST_1	EMMA_BHIF_SUB_INT_ST_1
#define EMMA_BHIF_INT_ST_2	EMMA_BHIF_SUB_INT_ST_2
#define EMMA_BHIF_INT_EN_0	EMMA_BHIF_SUB_INT_EN_0
#define EMMA_BHIF_INT_EN_1	EMMA_BHIF_SUB_INT_EN_1
#define EMMA_BHIF_INT_EN_2	EMMA_BHIF_SUB_INT_EN_2
#define EMMA_GPIO_INT_MASK	EMMA_GPIO_SUB_INT_MASK
#endif /* CONFIG_EMMA_SUBCPU */

/*
 * I/O helpers for EMMAXXX SoC register accesses.
 *
 * Note that 1) basically almost all EMMA registers are available
 * in 32-bit wide, and 2) we use __raw_ (non-swapped) accessors here
 * to avoid confusion sometimes caused by CONFIG_SWAP_IO_SPACE.
 */
#define emma_in(x)		__raw_readl((void __iomem *)&x)
#define emma_out(x, y)		__raw_writel((u32)y, (void __iomem *)&x)
#define emma_set_bits(x, mask)	  emma_out(x, emma_in(x) | mask)
#define emma_clear_bits(x, mask)  emma_out(x, emma_in(x) & ~mask)

#ifndef __ASSEMBLY__

#define EMMA_BASE 		(0xa0000000)

static inline void emma_sync(void)
{
	volatile u32 *p = (volatile u32 *)0xb0000000;
	(void)(*p);
	__asm__ __volatile__("sync");
}

static inline void emma_out32(u32 offset, u32 val)
{
	*(volatile u32 *)(EMMA_BASE | offset) = val;
	emma_sync();
}

static inline u32 emma_in32(u32 offset)
{
	u32 val = *(volatile u32 *)(EMMA_BASE | offset);
	return val;
}

static inline void emma_out16(u32 offset, u16 val)
{
	*(volatile u16 *)(EMMA_BASE | offset) = val;
	emma_sync();
}

static inline u16 emma_in16(u32 offset)
{
	u16 val = *(volatile u16 *)(EMMA_BASE | offset);
	return val;
}

static inline void emma_out8(u32 offset, u8 val)
{
	*(volatile u8 *)(EMMA_BASE | offset) = val;
	emma_sync();
}

static inline u8 emma_in8(u32 offset)
{
	u8 val = *(volatile u8 *)(EMMA_BASE | offset);
	return val;
}

#endif /* __ASSEMBLY__ */

#endif /* _ASM_EMMA_EMMAXXX_H */
